/* * * * * * * * * * * * * * * * * * * * * */
/*         TRSTAT - TfRemoteSTATus         */
/*      (c) OE3GMW & OE1HHC 24.09.93       */
/*                                         */
/*        Remote Support Programm          */
/*         fr  TFKISS und TFBAY           */
/* * * * * * * * * * * * * * * * * * * * * */

#include <ctype.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/*                                            lokale Funktionsprototypen  */
/**************************************************************************/
static  void        pascal banner(char near *);
static  unsigned    pascal fetch_driver_infos(void);
static  unsigned    pascal is_loaded(void);
static  void        pascal show_xdebug(unsigned char far *);


/*                                                      globale Variable  */
/**************************************************************************/
static unsigned     tfpcr_int;             /* der Host Interface Vektor   */
static unsigned     host_type;             /* Host Interface's id Byte    */
static void far    *defPar_ptr;            /* Zeiger zur  defPar  Struktur*/
static void far    *Debug_ptr;             /* Zeiger zur  Debug   Struktur*/


char near *pcr_txt[] = {"TFKISS", \
                        "Maximum load of RS-232 input buffer ... %u\n", \
                        "Maximum load of RS-232 out buffer ..... %u\n", \
                        "Maximum load of Host output buffer .... %u\n", \
                        "Number of RS-232 communication errors . %u\n", \
                        "Number of unexpected comm. interrupts . %u\n", \
                        ""};

char near *bay_txt[] = {"TFBAY ", \
                        "Maximum load of time interval buffer .. %u\n", \
                        "Maximum load of linear receive buffer . %u\n", \
                        "Maximum load of Host output buffer .... %u\n", \
                        "Number of lost TX interrupts .......... %u\n", \
                        "Number of unexpected comm. interrupts . %u\n", \
                        ""};

char near *non_txt[] = {"NONAME", \
                        "Value of Debug Register 0 ............. %u\n", \
                        "Value of Debug Register 1 ............. %u\n", \
                        "Value of Debug Register 2 ............. %u\n", \
                        "Value of Debug Register 3 ............. %u\n", \
                        "Value of Debug Register 4 ............. %u\n", \
                        ""};


/*                                                         here we start  */
/**************************************************************************/
void main(void)
{
  struct time t;
  struct date d;
  char near  *c;
  char near **txt;
  int far    *p;


  if (!(tfpcr_int = is_loaded())) {
    printf( \
      "    Fatal error. Hostmode interface not found!\r\n\n"\
      "    TRSTAT cannot be used as a standalone application program.\r\n"\
      "    Load hostmode interface driver first!\r\n\7");
    exit(1);
  }

  if (fetch_driver_infos()) {           /* Zeiger von Hostinterface holen */
    printf("\nUnprecedented response of hostmode interface!\n\7");
    exit(1);
  }

  switch (host_type & 0xF000) {
    case 0xE000 : txt = &pcr_txt[0];    /* Interface Programm ist TFKISS */
               break;
    case 0x4000 : txt = &bay_txt[0];    /* Interface Programm ist TFBAY   */
               break;
    default     : txt = &non_txt[0];    /* Interface Programm unbekannt   */
  }

  c = _argv[1];                         /* Auswertung der Kommandozeile   */
  if ((*c == '/') || (*c == '-'))
    c++;
  else
    c = "D";

  switch (toupper(*c)) {
    case '?':
    case 'H': printf( \
      "Usage:\n"\
      "    TRSTAT [[-?][-H][-R][-X]]\n\n"\
      " ?  or  H  this text.\n"\
      " R  reset %s's internal debug/status counters.\n"\
      " X  show extended status information,\n"\
      "    If there is no commandline parameter specified, the program\n"\
      "    will show %s's internal debug/status information.\n\n",\
      *txt, *txt);
              exit(0);

    case 'D': banner(*txt++);           /* Statusinformationen anzeigen   */
              p = Debug_ptr;
              printf( \
                "Debugging counters located at hex ..... %04X:%04X\n", \
                FP_SEG(p), FP_OFF(p));
              while (**txt)
                printf(*txt++, *p++);
              break;

    case 'R': banner(*txt++);           /* Statusinformationen rcksetzen */
              p = Debug_ptr;
              while (**txt) {
                txt++;
                *p++ = 0;
              }
              printf("Status counters cleared.\n");
              break;

    case 'X': banner(*txt);             /* Zusatzinfos, defPar dumpen     */
              show_xdebug(defPar_ptr);
              break;

    default:  printf( \
                "Illegal commandline option!  Use TRSTAT -H for Help.\n");
              exit(1);
  }
  gettime(&t);
  getdate(&d);
  printf("\nTRSTAT-timestamp: %2d.%02d.%02d - %2d:%02d:%02d\n\n", \
         d.da_day, d.da_mon, d.da_year, t.ti_hour, t.ti_min, t.ti_sec);
}


/*                                                  Kopfmeldung ausgeben  */
/**************************************************************************/
static void pascal banner(char near *p)
{
  printf("\n  TRSTAT - %s's remote status V 1.40"
         "\n  (c) OE3GMW & OE1HHC         %s"
         "\n\n", p, __DATE__);
}


/*                                       Show extended debug information  */
/**************************************************************************/
static void pascal show_xdebug(unsigned char far *p)
{
  char          i, j;
  unsigned char line[] = \
    "Offset  0  1  2  3  4  5  6  7    8  9  A  B  C  D  E  F" \
    "   0123456789ABCDEF";

  printf("System variables located at hex %04X:%04X. Dump:\n\n", \
          FP_SEG(p), FP_OFF(p));
  printf("%s\n", line);

  for (i = 0; i < 9; i++) {            /* 9 Zeilen zu je 16 Byte "dumpen" */
    sprintf(line, "+%02Xh:  ", i << 4);
    for (j = 0; j < 16; j++) {        /* 16 Byte 2-stelliger Hexdump      */
      sprintf(line + strlen(line), "%02X ", *(p + j));
      if (j == 7)
        strcat(line, "  ");           /* optische Trennung nach 8 Byte    */
    }
    strcat(line, "  ");

    for (j = 0; j < 16; j++, p++)     /* 16 Byte ASCII-Dump               */
      sprintf(line + strlen(line), "%c", isprint(*p) ? *p : '.');

    j = strlen(line) - 1;             /* rechtsbndige Spaces lschen     */
    while (line[j] == ' ')
      line[j--] = '\0';

    printf("%s\n", line);             /* ganze Zeile ausgeben             */
  }
}


/*                  Interrupttabelle nach installiertem Code durchsuchen  */
/**************************************************************************/
static unsigned pascal is_loaded(void)
{
  unsigned  i = 0x100;

  while (i-- > 0x2f) {
    if (!_fmemcmp(((char far *)getvect(i) + 3), "N5NX", 4)) {
      return(i);
    }
  }
  return(0);
}


/*                                   Informationen ber den Triber holen  */
/**************************************************************************/
static unsigned pascal fetch_driver_infos(void)
{
  struct  REGPACK cpu;

  cpu.r_ax = 0xFE00;                  /* Treibertype feststellen          */
  intr(tfpcr_int, &cpu);
  host_type = cpu.r_ax;

  cpu.r_ax = 0x500;                   /* Informationen ber Zeiger holen  */
  intr(tfpcr_int, &cpu);
  defPar_ptr = MK_FP(cpu.r_es, cpu.r_bx);
  Debug_ptr = MK_FP(cpu.r_es, cpu.r_dx);
  return(cpu.r_ax);
}


/* Ende von TRSTAT */
